#!/usr/bin/env python

import user, os, sys, getopt, exceptions, glob

def setup_env():
    if os.environ.has_key( "REPET_PATH" ):
        sys.path.append( os.environ.get( "REPET_PATH" ) )
    else:
        print "*** Error: no environment variable REPET_PATH"
        sys.exit(1)
setup_env()

from pyRepet.launcher.programLauncher import *

#------------------------------------------------------------------------------

def help():

    """
    Give the list of the command-line options.
    """

    print ""
    print "usage: %s [ options ]" % ( sys.argv[0] )
    print "options:"
    print "     -h: this help"
    print "     INPUT: use '-i' or '-I'"
    print "        -i: name of the input file (fasta format)"
    print "        -I: name of the input directory (containing fasta files)"
    print "     OUTPUT: use '-o' or '-O'"
    print "         -o: name of the output file (use only with '-i', default=stdout)"
    print "         -O: name of the output directory (use only with '-I', output file names being: prefix of input fasta file + '_shuffle.fa')"
    print "     -v: verbose (default=0/1/2)"
    print ""

#------------------------------------------------------------------------------

def main():

    """
    This program randomizes the sequence(s) contained in the input fasta file.
    It uses 'shuffle'.
    """

    inFileName = ""
    inDirName = ""
    outFileName = ""
    outDirName = ""
    verbose = 0

    try:
        opts, args = getopt.getopt(sys.argv[1:],"hi:I:o:O:v:")
    except getopt.GetoptError:
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-i":
            inFileName = a
        elif o == "-I":
            inDirName = a
        elif o == "-o":
            outFileName = a
        elif o == "-O":
            outDirName = a
        elif o == "-v":
            verbose = int(a)

    if inFileName == "" and inDirName == "":
        print "*** Error: missing input (file or directory)"
        help()
        sys.exit(1)

    if inDirName != "" and outDirName == "":
        print "*** Error: missing output directory"
        help()
        sys.exit(1)

    if verbose > 0:
        print "\nbeginning of %s" % ( sys.argv[0].split("/")[-1] )
        sys.stdout.flush()

    cwd = os.getcwd()

    if inFileName != "":
        pL = programLauncher( inFileName )
        pL.launchShuffle( outFileName )


    else:

        if os.path.exists( outDirName ):
            os.system( "rm -rf %s" % ( outDirName ) )
        os.mkdir( outDirName )

        fastaInDir = glob.glob( "%s/*.fa" %( inDirName ) )
        nbFasta = 1

        for i in fastaInDir:
            if verbose > 1:
                print "%3i / %3i" % ( nbFasta, len(fastaInDir) )
            pL = programLauncher( i )
            fastaPath, fastaName = os.path.split( i )
            prefix, extension = os.path.splitext( fastaName )
            pL.launchShuffle( "%s/%s_shuffle.fa" % ( outDirName, prefix ), verbose=verbose )
            nbFasta += 1

    if verbose > 0:
        print "%s finished successfully\n" % ( sys.argv[0].split("/")[-1] )
        sys.stdout.flush()

    return 0

#------------------------------------------------------------------------------

if __name__ == '__main__':
    main()
