#!/usr/bin/env python

import user, os, sys, getopt, exceptions
import ConfigParser

def setup_env():
    if "REPET_PATH" in os.environ.keys():
        sys.path.append( os.environ["REPET_PATH"] )
    else:
        print "*** Error: no environment variable REPET_PATH"
        sys.exit(1)
setup_env()

import pyRepet.sql.RepetDBMySQL
import pyRepet.sql.RepetJobMySQL

#-----------------------------------------------------------------------------

def help():

    print
    print "usage: ",sys.argv[0]," [ options ]"
    print "options:"
    print "     -h: this help"
    print "     -t: job table name (default='jobs')"
    print "     -j: job ID"
    print "     -g: group ID"
    print "     -q: queue name"
    print "     -s: new job status (waiting/running/error/finished)"
    print "     -m: method (file/db)"
    print "     -c: configuration file from TEdenovo or TEannot pipeline"
    print "     -H: MySQL host (if no configuration file)"
    print "     -U: MySQL user (if no configuration file)"
    print "     -P: MySQL password (if no configuration file)"
    print "     -D: MySQL database (if no configuration file)"
    print "     -T: MySQL port (if no configuration file, default=3306)"
    print

#-----------------------------------------------------------------------------

def main():

    tablename = "jobs"
    jobid = ""
    groupid = ""
    queue = ""
    newStatus = ""
    method = ""
    configFileName = ""
    host = ""
    user = ""
    passwd = ""
    dbname = ""
    port = 0

    try:
        opts, args = getopt.getopt(sys.argv[1:],"ht:j:g:q:s:m:c:H:U:P:D:T:")
    except getopt.GetoptError:
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
        elif o == "-t":
            tablename = a
        elif o == "-j":
            jobid = a
        elif o == "-g":
            groupid = a
        elif o == "-q":
            queue = a
        elif o == "-s":
            newStatus = a
        elif o == "-m":
            method = a
        elif o == "-c":
            configFileName = a
        elif o == "-H":
            host = a
        elif o == "-U":
            user = a 
        elif o == "-P":
            passwd = a
        elif o == "-D":
            dbname = a
        elif o == "-T":
            port = int(a)

    if  tablename == "" or jobid == "" or groupid == "" or queue == "" or newStatus == "":
        print "*** Error: missing compulsory options"
        help()
        sys.exit(1)

    if configFileName == "" and host == "" and user == "" and passwd == "" and dbname == "" and port == 0:
        print "*** Error: missing compulsory options"
        help()
        sys.exit(1)

    if configFileName != "":
        config = ConfigParser.ConfigParser()
        config.readfp( open(configFileName) )
        host = config.get("repet_env","repet_host")
        user = config.get("repet_env","repet_user")
        passwd = config.get("repet_env","repet_pw")
        dbname = config.get("repet_env","repet_db")
        port = config.get("repet_env","repet_port")

    #jobdb = pyRepet.sql.RepetJob.RepetJob( user, host, passwd, dbname )
    jobdb = pyRepet.sql.RepetJobMySQL.RepetJob( user, host, passwd, dbname, port )
    
    node = os.getenv( "HOSTNAME" )
    #job = pyRepet.sql.RepetJob.Job( tablename, jobid, groupid, queue, "", node )
    job = pyRepet.sql.RepetJobMySQL.Job( tablename, jobid, groupid, queue, "", node )
    
    print "current status: " + jobdb.get_job_status( job )
    sys.stdout.flush()

    jobdb.change_job_status( job, newStatus, method )

    print "updated status: " + jobdb.get_job_status( job )
    sys.stdout.flush()

    return 0

#----------------------------------------------------------------------------

if __name__ == "__main__":
    main()
