import re
import pyRepetUnit.hmmer.hmmOutput.HmmpfamOutput
import pyRepetUnit.hmmer.hmmOutput.HmmscanOutput
from pyRepet.util.file.FileUtils import *

class HmmOutputUtils:

    #--------------------------------------------------------------------------

    def __init__( self ):
        self.tabResult = []
        
    def isFirstFileEmpty(self, line):
        return line == ""    
     
    #--------------------------------------------------------------------------
    
    def writeHmmOutputToAlignFile( pfamOutput, fout ):        
            for item in pfamOutput.getList():
                for i in item:
                    fout.write(i + "\t")
                fout.write("0\n")    
            
    writeHmmOutputToAlignFile = staticmethod( writeHmmOutputToAlignFile )
    
    #--------------------------------------------------------------------------
        
    def readHmmpfamOutput( hmmerOutputFile ):
        """
        read an output from hmmpfam and return a array with results useful to build a .align file
        """
        #Tested with HMMER 2.3.2 on Linux (Debian) and on Unix (Solaris)
        line = hmmerOutputFile.readline()
        tabResult = None
        aRecup = 0        
        if (line == ""):            
            tabResult = None            
            return tabResult
        tabResult = pyRepetUnit.hmmer.hmmOutput.HmmpfamOutput.HmmpfamOutput()
        while not re.match("Alignments of top-scoring domains:.*", line):
            line = hmmerOutputFile.readline()                   
            m = re.match("Query sequence:\s*(.*)", line)
            if m:                    
                seqName = m.group(1)
            #warning : do no use a return character because they change between several OS                                              
            if re.match("--------\s+-------\s+-----\s+-----\s+-----\s+-----\s+-----\s+-------.*", line):
                aRecup = 1                
            m = re.match("(\S+)\s+\d+\/\d+\s+(\d+)\s+(\d+)\s+.+\s+(\d+)\s+(\d+)\s+.+\s+(.+\.\d)\s+(.+)", line)
            if m and aRecup == 1:                
                tabResult.append([seqName, m.group(2), m.group(3), m.group(1), m.group(4), m.group(5), m.group(7), m.group(6)]) 
            if line == "":            
                tabResult = None            
                return tabResult
        return tabResult  
    
    readHmmpfamOutput = staticmethod( readHmmpfamOutput )
    
    #--------------------------------------------------------------------------
    
    def readHmmscanOutput( hmmerOutputFile ):
        """
        read an output from hmmscan and return a array with results useful to build a .align file
        """
        #Tested with HMMER 3.a2 on Linux
        line = hmmerOutputFile.readline()
        tabResult = None
        aRecup = 0        
        if (line == ""):            
            tabResult = None            
            return tabResult
        tabResult = pyRepetUnit.hmmer.hmmOutput.HmmscanOutput.HmmscanOutput()
        
        while not re.match("Internal pipeline statistics summary:.*", line):
            line = hmmerOutputFile.readline()                
            m = re.match("Query:\s+(.*?)\s+.*", line)
            if m:                    
                seqName = m.group(1)                
            m = re.match(">>\s(.+?)\s+.*", line)
            if m:                    
                profilName = m.group(1)                
            if re.match("\s+---\s+------\s+-----\s+---------\s+---------\s+-------\s+-------\s+-------\s+-------\s+-------\s+-------\s+----.*", line):    
                aRecup = 1                 
            m = re.match("\s+\d+\s+.+?\s+(.+?)\s+.+?\s+.+?\s+(.+?)\s+(\d+)\s+(\d+)\s+.+?\s+(\d+)\s+(\d+)\s+.+?\s+\d+\s+\d+\s+.+?\s+.+?", line)
            if m and aRecup == 1: 
                tabResult.append([seqName, m.group(5), m.group(6), profilName, m.group(3), m.group(4), m.group(2), m.group(1)]) 
            if line == "":            
                tabResult = None            
                return tabResult
        return tabResult  
    
    readHmmscanOutput = staticmethod( readHmmscanOutput )
    
    #--------------------------------------------------------------------------
    
    
if __name__ == "__main__":                 
    main()        